#!/usr/bin/env node
"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const Server_1 = require("./Server");
const services = require("./services");
const logger = require("./shared/Logger");
const fs = require("fs");
const program = require("commander");
const path = require("path");
const tmp = require("tmp");
const child_process_1 = require("child_process");
function tmpRunMigrationStage(stage, args, options) {
    return __awaiter(this, void 0, void 0, function* () {
        const cmd = path.resolve(__dirname, "../dist/migration", stage);
        const tmpdir = tmp.dirSync({ unsafeCleanup: true });
        try {
            const child = child_process_1.fork(cmd, args, Object.assign({}, options, { cwd: tmpdir.name }));
            yield new Promise((resolve, reject) => {
                child.on('exit', (code) => {
                    if (code) {
                        throw new Error(`command failed: ${cmd}`);
                    }
                    resolve();
                });
            });
        }
        finally {
            tmpdir.removeCallback();
        }
    });
}
class Cli {
    constructor() {
        this.program = new program.Command();
    }
    migrateCommand(args) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!args.from || !args.to) {
                console.error("Please specify both --from and --to. Aborting.");
                this.program.outputHelp();
                process.exit(1);
            }
            yield tmpRunMigrationStage("from1", [path.resolve(args.from)]);
            const to2args = ['--from', path.resolve(args.from), '--to', path.resolve(args.to)];
            if (args.copyrealms) {
                to2args.push('--copyrealms');
            }
            yield tmpRunMigrationStage("to2", to2args);
        });
    }
    start(argv) {
        this.program.version(require('../package.json').version);
        this.program.command('migrate')
            .description('Migrate data from Realm Object Server 1.x format to 2.0 format. Existing 1.x data is preserved.')
            .option("-f, --from <path>", "The source root directory (ROS-1.x)")
            .option("-t, --to <path>", "The destination root directory (ROS-2.x)")
            .option("-r, --copyrealms", "Also copy the user realms. Without this parameter, only meta data is migrated.")
            .action((command) => __awaiter(this, void 0, void 0, function* () {
            yield this.migrateCommand(command);
            process.exit(0);
        }));
        this.program.on('--help', () => {
            console.log('');
            console.log('  Help for individual commands:');
            console.log('');
            console.log('    ros start --help');
            console.log('    ros migrate --help');
            console.log('');
        });
        this.program
            .command('start')
            .description('Start Realm Object Server')
            .option("-d, --data <path>", "Specify the data path to use (default ./data)", "./data")
            .option("-a, --address <address>", "Specify the address to listen on (default 0.0.0.0)", "0.0.0.0")
            .option("-p, --port <port>", "Specify the port to listen on (default 9080)", "9080")
            .option("-l, --loglevel <level>", "The log level (all | trace | debug | detail | info | warn | error | fatal | off) (default info)", "info")
            .option("-f, --logfile <filename>", "The file to log to. If the option is absent, logging will be directed to stdout")
            .option("--no-auto-keygen", "Do not autogenerate keys on startup")
            .option("--private-key <privateKeyPath>", "Alternative path to the private key (default <data>/keys/auth.key)")
            .option("--public-key <publicKeyPath>", "Alternative path to the public key (default <data>/keys/auth.pub)")
            .option("--refresh-token-ttl <refreshTokenTtl>", "TTL of issued refresh tokens in seconds", String(10 * 60 * 60 * 24 * 365 * 10))
            .option("--access-token-ttl <accessTokenTtl>", "TTL of issued access tokens in seconds", String(10 * 60))
            .option('--auth <provider1[,provider2 ...]>', "A list of auth providers to load", "password")
            .action((command) => {
            const dataPath = path.resolve(command.data);
            if (fs.existsSync(dataPath)) {
                const stat = fs.statSync(dataPath);
                if (!stat.isDirectory()) {
                    console.error("The data directory '%s' is not a directory. Aborting", dataPath);
                    process.exit(1);
                }
                try {
                    fs.accessSync(dataPath, fs.constants.R_OK | fs.constants.W_OK | fs.constants.X_OK);
                }
                catch (e) {
                    console.error("The data directory '%s' does not have 'rwx' permissions. Aborting", dataPath);
                    process.exit(1);
                }
            }
            else {
                try {
                    fs.mkdirSync(dataPath, 0o700);
                }
                catch (e) {
                    console.error("The data directory '%s' cannot be created. Aborting", dataPath);
                    process.exit(1);
                }
            }
            const port = parseInt(command.port, 10);
            if (!(String(port) === command.port && port > 0 && port < 65536)) {
                console.error("Invalid port number (1-65535): '" + command.port + "'. Aborting.");
                process.exit(1);
            }
            let loglevelOk = false;
            for (const level in logger.SyncLogLevel) {
                if (command.loglevel === logger.SyncLogLevel[level]) {
                    loglevelOk = true;
                    break;
                }
            }
            if (!loglevelOk) {
                console.error("Invalid loglevel '%s'. Aborting.", command.loglevel);
                process.exit(1);
            }
            if (command.logfile) {
                try {
                    fs.openSync(command.logfile, 'a');
                }
                catch (e) {
                    console.error("The log file '%s' cannot be opened for writing. Aborting", command.logfile);
                    process.exit(1);
                }
            }
            if (command.privateKey) {
                try {
                    fs.openSync(command.privateKey, 'r');
                }
                catch (e) {
                    console.error("The private key '%s' does not exist or is not readable. Aborting", command.privateKey);
                    process.exit(1);
                }
            }
            if (command.publicKey) {
                try {
                    fs.openSync(command.publicKey, 'r');
                }
                catch (e) {
                    console.error("The public key '%s' does not exist or is not readable. Aborting", command.publicKey);
                    process.exit(1);
                }
            }
            const refreshTokenTtl = parseInt(command.refreshTokenTtl, 10);
            if (String(refreshTokenTtl) !== command.refreshTokenTtl || refreshTokenTtl <= 0) {
                console.error("refresh-token-ttl must be a positive number. Aborting");
                process.exit(1);
            }
            const accessTokenTtl = parseInt(command.accessTokenTtl, 10);
            if (String(accessTokenTtl) !== command.accessTokenTtl || accessTokenTtl <= 0) {
                console.error("access-token-ttl must be a positive number. Aborting");
                process.exit(1);
            }
            const config = {
                hostname: command.address,
                port: port,
                logLevel: command.loglevel,
                dataPath: dataPath,
                privateKeyPath: command.privateKey ? path.resolve(command.privateKey) : null,
                publicKeyPath: command.publicKey ? path.resolve(command.publicKey) : null,
                autoKeyGen: command.autoKeygen,
            };
            if (command.logfile) {
                config.logger = new logger.FileConsole(command.logfile, command.loglevel);
            }
            else {
                config.logger = new logger.Console(command.loglevel);
            }
            const authService = new services.AuthService({
                refreshTokenTtl: refreshTokenTtl,
                accessTokenTtl: accessTokenTtl
            });
            const providerNames = command.auth.split(',');
            for (const providerName of providerNames) {
                let providerClass;
                try {
                    providerClass = require(`./auth/${providerName}`).default;
                }
                catch (e) {
                    console.error("The auth provider '%s' cannot be found or does not have a default export", providerName);
                    process.exit(1);
                }
                const options = {};
                if (providerName === "password") {
                    options.autoCreateAdminUser = true;
                }
                const provider = new providerClass(options);
                authService.addProvider(provider);
            }
            const server = new Server_1.Server(config);
            server.addService(authService);
            server.addService(new services.RealmDirectoryService());
            server.addService(new services.LogService());
            server.addService(new services.SyncProxyService());
            server.addService(new services.SyncService());
            server.addService(new services.HealthService());
            server.addService(new services.PermissionService());
            process.removeAllListeners("SIGINT");
            process.on('SIGINT', () => __awaiter(this, void 0, void 0, function* () {
                config.logger.info("Got signal SIGINT.  Shutting down");
                server.shutdown()
                    .then(() => process.exit(0))
                    .catch((err) => {
                    console.error(err);
                    process.exit(1);
                });
            }));
            process.removeAllListeners("SIGTERM");
            process.on('SIGTERM', () => __awaiter(this, void 0, void 0, function* () {
                config.logger.info("Got signal SIGTERM.  Shutting down.");
                server.shutdown()
                    .then(() => process.exit(0))
                    .catch((err) => {
                    console.error(err);
                    process.exit(1);
                });
            }));
            process.on("unhandledRejection", (reason, promise) => {
                console.error("Detected an unhandled promise rejection at: ", promise, ", reason: ", reason);
            });
            server.start().catch(err => console.error(err));
        });
        if (!argv.slice(2).length) {
            this.program.outputHelp();
            process.exit(1);
        }
        return this.program.parse(argv);
    }
}
exports.Cli = Cli;
if (!module.parent) {
    new Cli().start(process.argv);
}
//# sourceMappingURL=cli.js.map